/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.rational.report.web.core.app.internal.ReportApplication");

dojo.require("net.jazz.ajax.ui._Application");
dojo.require("net.jazz.ajax.ui.PageContainer");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.rational.report.web.core.app.internal.Header");
dojo.require("com.ibm.rational.report.web.core.app.internal.Navbar");
dojo.require("com.ibm.rational.report.web.core.app.internal.Footer");
dojo.require("com.ibm.rational.report.web.core.client.internal.SessionManager");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");

var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var InternalMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");
var PageContainer			= net.jazz.ajax.ui.PageContainer;
var PlatformUI				= net.jazz.ajax.ui.PlatformUI;
var Header					= com.ibm.rational.report.web.core.app.internal.Header;
var Navbar					= com.ibm.rational.report.web.core.app.internal.Navbar;
var Footer					= com.ibm.rational.report.web.core.app.internal.Footer;
var SessionManager			= com.ibm.rational.report.web.core.client.internal.SessionManager;

/**
 * The abstract superclass that provides a common look and feel for
 * Vega Reportal web applications.
 * 
 * Subclasses must minimally do the following to extend this class
 * 1) Define an 'init' method to perform whatever initialization logic
 *    they require; this method should call 'this._initUI()' at some
 *    point to produce the standard look and feel.
 * 2) Call the 'start' method of this class via the extender's
 *    superclass' property;
 *    e.g. my.extender.superclass.start.call(this);
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.internal.ReportApplication",
	net.jazz.ajax.ui._Application, {
	
	start: function() {
		ReportApplication.superclass.start.call(this);
	},
	
	_initUI: function(args) {
		PlatformUI.getWorkbench()._setDefaultTitle(this.getHTMLTitle());		
		if (!args.label || !dojo.trim(args.label)) {
			var msg = dojo.string.substitute(
				ServiceabilityMessages.INTERNAL_APPLICATION_ERROR_WITH_REASON, [InternalMessages.NO_VALID_PROJECT_AREA]);
			throw new Error(msg);
		}
		args.label = dojo.trim(args.label);
		
		// remove unwanted pages
		dojo.forEach(net.jazz.ajax.ui.internal.PageRegistry.getPages(), function(page) {
			if (page.removalPropertyId != null) {
				var property = com.ibm.team.repository.web.client.init.InitializationData.get(page.removalPropertyId);
				if (property != null && property.value != undefined) {
					dojo.setObject(page.removalPropertyId, property.value);
				}
			}
		});
		
		// order pages
		var orderedPageIDs = getOrderedPageIDs();
		if (orderedPageIDs) {
			PlatformUI.getWorkbench().orderPages(function(a, b) {
				return sortPages(a, b, orderedPageIDs);
			});
		}
		
		// for min-width
		dojo.addClass(dojo.body(), "com-ibm-rational-report-web com-ibm-team-repository-web");
		dojo.addClass(workbenchRootNode(), "com-ibm-team-repository-web-RootNode");
		
		// create the main body elements
		var suppressedPages = getSuppressedPages();
		if (suppressedPages) {
			args.suppressedPages = suppressedPages;
		}
		
		//getWorkbench().showMessage("Loading...");
		workbenchRootNode().appendChild((this.getHeaderWidget(args)).domNode);
		
		var pageArea = new PageContainer();
		workbenchRootNode().appendChild(pageArea.domNode);
		pageArea.domNode.insertBefore((this.getNavbarWidget(args)).domNode, pageArea.domNode.firstChild);
		
		// append footer directly to body to make it stick to the bottom
		dojo.body().appendChild((this.getFooterWidget()).domNode);
		
		SessionManager.startTimeout();
	},
	
	getHTMLTitle: function(args) {
		return "Rational Common Reporting";
	},
	
	getHeaderWidget: function(args) {
		return new Header(args);
	},
	
	getNavbarWidget: function(args) {
		return new Navbar(args);
	},	
	
	getFooterWidget: function() {
		return new Footer();
	}
	
});

// aliases
var ReportApplication		= com.ibm.rational.report.web.core.app.internal.ReportApplication;

// utility functions
function getCurrentWorkbenchID() {
	return net.jazz.ajax.ui.PlatformUI._getApplicationId();
}

function workbenchRootNode() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode();
}

function getSuppressedPages() {
	if (!com.ibm.team.repository.web.suppressedPages) {
		return null;
	}
	
	var suppressedPageSets = dojo.fromJson(com.ibm.team.repository.web.suppressedPages);
	
	if (!dojo.isObject(suppressedPageSets)) {
		var reason = InternalMessages.JSON_SYNTAX_ERROR +
			" (raw JSON: " + com.ibm.team.repository.web.suppressedPages + ")";
		var msg = dojo.string.substitute(
			ServiceabilityMessages.INVALID_PROPERTY_VALUE, ['com.ibm.team.repository.web.suppressedPages', reason]);
		console.debug(msg);
		return null;
	}
	
	var suppressedPageList = suppressedPageSets[getCurrentWorkbenchID()];
	
	var badArray = false;
	if (!dojo.isArray(suppressedPageList)) {
		badArray = true;
	} else {
		for (var i = 0; i < suppressedPageList.length; i++) {
			if (!dojo.isString(suppressedPageList[i])) {
				badArray = true;
				break;
			}
		}	
	}
	if (badArray) {
		var reason = dojo.string.substitute(
			InternalMessages.SUPPRESSED_PAGE_LIST_SYNTAX_ERROR, [getCurrentWorkbenchID()]) +
				" (raw JSON: " + com.ibm.team.repository.web.suppressedPages + ")";
		var msg = dojo.string.substitute(
			ServiceabilityMessages.INVALID_PROPERTY_VALUE, ['com.ibm.team.repository.web.suppressedPages', reason]);
		console.debug(msg);
		return null;
	}
	
	return suppressedPageList;
}

/**
 * Parses out ordered page ids from the config property
 * example of valid JSON:
 *  {"com.ibm.team.repository.web.projectArea":
 *  ["com.ibm.team.workitem", "com.ibm.team.apt", "com.ibm.team.reports"]}
 */
function getOrderedPageIDs() {
	if (!com.ibm.team.repository.web.pageorder) {
		return null;
	}
	
	var orderSets = dojo.fromJson(com.ibm.team.repository.web.pageorder);
	
	if (!dojo.isObject(orderSets)) {
		// if we didn't get an object back, the property wasn't valid JSON
		// log it and move on
		var reason = InternalMessages.JSON_SYNTAX_ERROR +
			" (raw JSON: " + com.ibm.team.repository.web.pageorder + ")";
		var msg = dojo.string.substitute(
			ServiceabilityMessages.INVALID_PROPERTY_VALUE, ['com.ibm.team.repository.web.pageorder', reason]);
		console.debug(msg);
		return null;
	}
	 
	var orderedPageIDs = orderSets[getCurrentWorkbenchID()];
	if (!orderedPageIDs){
		// early return if we don't have
		return null;
	}
	
	var badArray = false;
	if (!dojo.isArray(orderedPageIDs)) {
		badArray = true;
	} else {
		for (var i = 0; i < orderedPageIDs.length; i++) {
			if (!dojo.isString(orderedPageIDs[i])) {
				badArray = true;
				break;
			}
		}	
	}
	if (badArray) {
		var reason = dojo.string.substitute(
			InternalMessages.SUPPRESSED_PAGE_LIST_SYNTAX_ERROR, [getCurrentWorkbenchID()]) +
				" (raw JSON: " + com.ibm.team.repository.web.pageorder + ")";
		var msg = dojo.string.substitute(
			ServiceabilityMessages.INVALID_PROPERTY_VALUE, ['com.ibm.team.repository.web.pageorder', reason]);
		console.debug(msg);
		return null;
	}
	
	return orderedPageIDs;
}

function sortPages(a, b, orderedPageIDs) {
	for (var i = 0; i < orderedPageIDs.length; i++) {
		if(orderedPageIDs[i] === a.id) {
			return -1;
		}
		if (orderedPageIDs[i] === b.id) {
			return 1;
		}
	}
	return -1;
}

})();
